data State = Dead | Alive

type Cell = (Int, Int)

cN :: Int
cN = 8

cells :: [Cell]
cells = [(i,j) | i<-[0..cN-1], j<-[0..cN-1]]

neighbors :: Cell -> [Cell]
neighbors(i,j) = [ (i',j') | i' <- [i-1..i+1],  
                             j' <- [j-1..j+1],
                             (i,j) /= (i',j'),  
                             valid i', valid j' ] 
  where valid i = i `elem` [0..cN-1]

aliveNeighbors :: Cell -> Int
aliveNeighbors cell =  
    length([ c | c <- neighbors cell, status c == Alive ])

status :: Dynamic (Cell -> State)
status = initAssocs "status" [ (c,Dead) | c <- cells ]

instance AsmTerm State

letDie :: Cell -> Int -> Rule   ()
letDie cell n =
  if status cell == Alive && (n < 2 || n > 3) then 
    status cell := Dead
  else skip

letLive :: Cell -> Int -> Rule   ()
letLive cell n =
  if status cell == Dead && n==3 then
    status cell := Alive
  else skip

gameOfLife :: Rule   ()
gameOfLife = 
   forall cell <- cells do
     let n = aliveNeighbors cell
     letDie cell n
     letLive cell n
